from .BasicHeuristics import BasicHeuristics
from .ReCoRDHeuristics import ReCoRDHeuristics
from .WordInContextHeuristics import WordInContextHeuristics

__all__ = [
    "BasicHeuristics",
    "ReCoRDHeuristics",
    "WordInContextHeuristics",
]